# blankfilter.awk -- a noweb notangle filter
#   The filter does the following:
#   - it removes superfluous indentation blanks from noweb code chunks
#   - it removes blank lines before documentation chunks

BEGIN \
{
  true = 1;
  false = 0;
  inCodeChunk = false;
  bufferLength = 0;
}

#--------------------

/.*/ \
{
  currentLine = $0;
}

#--------------------

/^@end code/ \
{
  inCodeChunk = false;
  flushBuffer();
  print currentLine;
  next;
}

#--------------------

/^@begin code/ \
{
  inCodeChunk = true;
  flushBuffer();
  buffer[++bufferLength] = currentLine;
  next;
}

#--------------------

/.*/ && !inCodeChunk \
{
  print currentLine;
  next;
}

#--------------------

/^@text/ && inCodeChunk \
{
  effectiveText = substr(currentLine, 7, 1000);

  if (substr(effectiveText, 1, 2) == "  ") {
     effectiveText = substr(effectiveText, 3, 1000);
  }

  currentLine = "@text " effectiveText;
  buffer[++bufferLength] = currentLine;
  next;
}

#--------------------

/.*/ && inCodeChunk \
{
  buffer[++bufferLength] = currentLine;
  next;
}

#--------------------

END \
{
  flushBuffer();
}

#--------------------
#--------------------

function flushBuffer() \
{
  # remove trailing blank lines
  for (i = bufferLength - 1;  i > 0;  i--) {
    thisLine = buffer[i];

    if (thisLine ~ /^@text *$/ || thisLine ~ /^@nl$/ ) {
      bufferLength--;
    } else {
      break;
    }
  }

  for (i = 1;  i <= bufferLength;  i++) {
    print buffer[i];
  }

  bufferLength = 0;
}
